//========= Copyright  1996-2001, Valve LLC, All rights reserved. ============
//
// Purpose: 
//
// $NoKeywords: $
//=============================================================================

// Triangle rendering, if any

#include "hud.h"
#include "cl_util.h"

// Triangle rendering apis are in gEngfuncs.pTriAPI

#include "const.h"
#include "entity_state.h"
#include "cl_entity.h"
#include "triangleapi.h"
#include "pmtrace.h"
#include "pm_defs.h"

#include "event_api.h"



#define DLLEXPORT __declspec( dllexport )

extern "C"
{
	void DLLEXPORT HUD_DrawNormalTriangles( void );
	void DLLEXPORT HUD_DrawTransparentTriangles( void );
};

/*
=================
Draw_Triangles

Example routine.  Draws a sprite offset from the player origin.
=================
*/
void Draw_Triangles( void )
{
        for ( int i=1; i < 700; i++)        
        {
                cl_entity_t *player = gEngfuncs.GetEntityByIndex(i);
                vec3_t org;
                int size;

                if ( !player )
                        return;

                int m_ihgrunt = gEngfuncs.pEventAPI->EV_FindModelIndex( "models/hgrunt.mdl" );
                int m_ispike = gEngfuncs.pEventAPI->EV_FindModelIndex( "models/spike.mdl" );
                int m_ihead = gEngfuncs.pEventAPI->EV_FindModelIndex( "models/hgrunthead.mdl" );
                if (
                        ( player->curstate.modelindex != m_ihgrunt ) &&
                        ( player->curstate.modelindex != m_ihead ) &&
                        ( player->curstate.modelindex != m_ispike )
                   )
                        continue;

                if (player->player)
                        continue;

                if (player == gEngfuncs.GetLocalPlayer())
                        continue;

                //this essentially filters out dead entities.
                //thanks to darkangelzt @ wavelength forums
                if (player->curstate.messagenum < gEngfuncs.GetLocalPlayer()->curstate.messagenum)
                        continue;

                if ( player->curstate.modelindex == m_ihgrunt )
                        size = 32;
                else if ( player->curstate.modelindex == m_ispike )
                        size = 4;
                else if ( player->curstate.modelindex == m_ihead )
                {
                        //gEngfuncs.Con_DPrintf( "m_ihead\n" );
                        size = 12;
                }


                org = player->origin;
        
                pmtrace_t *tr;                                                         //PHYSENTSONLY ANYVISIBLE         ignore
                if ( player->curstate.modelindex == m_ihead )
                        tr = gEngfuncs.PM_TraceLine( player->origin + Vector(0,0,-7), Vector(0,0,-2048), PM_TRACELINE_PHYSENTSONLY | PM_STUDIO_IGNORE, 2 /*point sized hull*/, -1 );
                else
                        tr = gEngfuncs.PM_TraceLine( player->origin, Vector(0,0,-2048), PM_TRACELINE_PHYSENTSONLY, 2 /*point sized hull*/, -1 );
        
                //bc
                org.z = tr->endpos.z + 1;
        
                
        
                if (gHUD.m_hsprCursor == 0)
                {
                        char sz[256];
                        sprintf( sz, "sprites/shadow.spr" );
                        gHUD.m_hsprCursor = SPR_Load( sz );
                }
        
                if ( !gEngfuncs.pTriAPI->SpriteTexture( (struct model_s *)gEngfuncs.GetSpritePointer( gHUD.m_hsprCursor ), 0 ))
                {
                        return;
                }
         
                // Create a triangle, sigh
                gEngfuncs.pTriAPI->RenderMode( kRenderTransTexture );
                gEngfuncs.pTriAPI->CullFace( TRI_NONE );
                gEngfuncs.pTriAPI->Begin( TRI_QUADS );
                // Overload p->color with index into tracer palette, p->packedColor with brightness
                gEngfuncs.pTriAPI->Color4f( 1.0, 1.0, 1.0, 1.0 );
                // UNDONE: This gouraud shading causes tracers to disappear on some cards (permedia2)
        
                gEngfuncs.pTriAPI->Brightness( 1 );
                gEngfuncs.pTriAPI->TexCoord2f( 0, 0 );
                gEngfuncs.pTriAPI->Vertex3f( org.x - size, org.y - size, org.z );
        
                gEngfuncs.pTriAPI->Brightness( 1 );
                gEngfuncs.pTriAPI->TexCoord2f( 0, 1 );
                gEngfuncs.pTriAPI->Vertex3f( org.x - size, org.y + size, org.z );
        
                gEngfuncs.pTriAPI->Brightness( 1 );
                gEngfuncs.pTriAPI->TexCoord2f( 1, 1 );
                gEngfuncs.pTriAPI->Vertex3f( org.x + size, org.y + size, org.z );
        
                gEngfuncs.pTriAPI->Brightness( 1 );
                gEngfuncs.pTriAPI->TexCoord2f( 1, 0 );
                gEngfuncs.pTriAPI->Vertex3f( org.x + size, org.y - size, org.z );
        
                gEngfuncs.pTriAPI->End();
                gEngfuncs.pTriAPI->RenderMode( kRenderNormal );
        }
}

/*
=================
HUD_DrawNormalTriangles

Non-transparent triangles-- add them here
=================
*/
void DLLEXPORT HUD_DrawNormalTriangles( void )
{
}

/*
=================
HUD_DrawTransparentTriangles

Render any triangles with transparent rendermode needs here
=================
*/
void DLLEXPORT HUD_DrawTransparentTriangles( void )
{
      Draw_Triangles();
}
